import textwrap
import smtplib
import requests
import click
import bs4
import os

from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

email_template = """Dear {name},

I hope this message finds you well. I am writing to bring to your attention an important matter regarding the usage of nodes on our shared platform, Infinity.

Infinity is designed as a collaborative environment, where all users rely on each other's cooperation to ensure everyone has access to the necessary resources. I have noticed that you may be an overutilizing compute nodes currently.

We kindly request you to review your current jobs and verify if there's an opportunity to optimize your resource usage. For instance, you might consider consolidating your work to use fewer nodes at a time, where feasible.
This will not only help in maintaining the platform's efficiency but also ensure that all users have equitable access to the resources.

Please understand that I am sending this email to keep the integrity and functionality of our shared platform as a collective effort.

Should you have any questions or require assistance in managing your resource allocation, please do not hesitate to reach out. We are more than happy to help.
Best regards,

{complain_name}.
"""

url = "https://infinity-users.projet-horizon.fr/index.php"

@click.group()
def main():
  pass

def get_data():
    with requests.session() as s:
        response = s.post(url, data={"authpw": "astrorizon"})
        if response.status_code != 200:
            print("Connection failure")
            raise SystemExit(1)

        r = s.get(url, params={"n": "Site.CurrentUsers"})
        soup = bs4.BeautifulSoup(r.text, "html.parser")
        users = soup.find("pre")

        # Define an empty list to store user data as dictionaries
        users_tbl = []

        # Iterate through each row in the table and add the data as a dictionary to users list
        for row in users.get_text().split("\n")[3:]:
            if len(row.strip()) == 0:
                continue
            # Split each row into columns using space as delimiter
            columns = list(map(lambda x: x.strip(), row.split(" | ")[:]))

            # Initialize an empty dictionary with keys as column headers
            user_data = {
                "name": "",
                "status": "",
                "last_login": "",
                "email": "",
                "login": "",
                "mentor": "",
            }    

            # Assign values to each key in the dictionary from corresponding columns
            (
                user_data["login"],
                user_data["name"],
                user_data["status"],
                user_data["last_login"],
                user_data["email"],
                user_data["mentor"],
            ) = columns
            user_data["mentor"] = user_data["mentor"][:-1].strip()
            user_data["login"] = user_data["login"][1:].strip()
            # Append the populated dictionary to users list
            users_tbl.append(user_data)

        # Print the parsed data
        mapped_login = {user["login"]: user for user in users_tbl}
        return mapped_login

@main.command()
@click.argument("user")
def sponsor(user):
    if user is None:
        print("Please provide the login of the user to bug")
        raise SystemExit(1)

    mapped_login = get_data()
   
    u = mapped_login[user] 

    print(u["mentor"])
    print(mapped_login[u["mentor"]]["name"])

@main.command()
@click.argument("to_bug")# help="The login of the user to bug")
def go(to_bug):
    """Bug a user on the Infinity platform.
    
    Args:
        to_bug (str): The login of the user to bug.
    """
    if to_bug is None:
        print("Please provide the login of the user to bug")
        raise SystemExit(1)

    mapped_login = get_data()
 
    # Get the current unix login of the user using python os
    current_login = os.getlogin()

    if not current_login in mapped_login:
        print(f"User {current_login} not found")
        raise SystemExit(1)
    
    from_user = mapped_login[current_login]
    if not to_bug in mapped_login:
        print(f"User {to_bug} not found")
        raise SystemExit(1)
    
    to_user = mapped_login[to_bug]

    email_body = email_template.format(
        name=to_user["name"], complain_name=from_user["name"]
    )

    # Create a multipart message
    msg = MIMEMultipart()
    msg["From"] = from_user["email"]
    msg["To"] = to_user["email"]
    msg["CC"] = from_user["email"]
    msg["Subject"] = "Your usage of Infinity nodes"
    msg.attach(MIMEText(email_body, "plain"))

    text = msg.as_string()
    print(textwrap.dedent(f"""
Here is the draft of the email that is going to be sent:
-----------------------------------------------------
{text}
-----------------------------------------------------

If you agree with it, please enter your login name to confirm the sending of the email.
"""))

    if input("Do you want to send the email? ") != current_login:
        print("Email not sent")
        raise SystemExit(0)

    print("Sending email...")
    server = smtplib.SMTP('smtp.iap.fr', 587)
    server.sendmail(from_user["email"], [to_user["email"], from_user["email"]], text)
    server.quit()

if __name__ == "__main__":
    main()
